#ifndef _CCANNODE_H
#define _CCANNODE_H


#include <string>
#include <map>
using namespace std;


class CCanMsgId;
class CUsbPicCan;

/// Class to interact with a unique physical CAN node. Abstract class.
/**
A CAN node is an electronic board that is able to receive and send messages
on the CAN bus. There may be different types of nodes, and several nodes of
the same type, identified by a unique MAC address (#macId).

This class must be derived as many times as there are node types on the CAN bus.
Then, each subclass must be instantiated as many times as there are nodes of its
type on the network.

When the USB-CAN gateway receives a CAN message with CUsbPicCan::canRcv(), it
searches the destinee among a list of CCanNode objects, based on their MAC
address. It then calls #notify() to let the object know that a message was
received for it.

The derived classes must implement the protected function #deliver(), whose
purpose is to process incoming CAN messages.
*/
class CCanNode
{
  public:
    /// CAN message.
    /**
    Map to CUsbPic::SBPacket with \a command = CAN_MSG and \a len =  sizeof(SCanMsg-2).
    */
    struct __attribute__ ((__packed__)) SCanMsg
    {
    	char usbCmd;		///< Used for data alignment : SBPacket::command. Don't use.
    	char usbSize;		///< Used for data alignment : SBPacket::len. Set to sizeof(TCanMsg)-2.
    	short id;		///< 11-bit CAN message ID in a 16-bit word, right justified.
    	char len;		///< Number of bytes in \a data.
    	char data[8];		///< CAN message data.
    	char isRtr;		///< Non-zero if message is a remote frame, zero if data frame.
    	char prio;		///< Message priority from 0 (lowest) to 3 (highest).
    };

    union __attribute__ ((__packed__)) UCanMsg
    {
    	char _byte[15];
    	SCanMsg message;

    };

    typedef UCanMsg TCanMsg;

	CCanNode(short macAddress);
	unsigned short get_macId();
	~CCanNode();
	void set_macId(unsigned short value);
	void notify(const TCanMsg & msg);
	void dataConsummed();
	static short identify(const TCanMsg & msg);
	void addMessage(const string & name, const CCanMsgId & message);
	void set_gateway(CUsbPicCan *value);
	bool get_hasNewData();

  protected:
	unsigned short macId;		///< CAN node MAC address.
	TCanMsg msgRcv;		///< CAN message received.
	TCanMsg msgSnd;		///< CAN message to send.
	bool hasNewData;		///< True if new data received from CAN. Set by notify() ; must be reset by GUI.
	/// Delivers an incoming message to the properties of the object.
	/**
	The implementations of this function must extract the data out of an incoming
	CAN message and write them to the class attributes.

	Once #notify() has been called by the USB-CAN gateway, the CCanNode-derived
	object doesn't know which message it has received. This is why #notify() calls
	#deliver() whose task is :
	- loop through the possible messages (usually only a handful)
	- extract the expected data out of the message
	- set the object properties
	*/
	virtual void deliver() = 0;
	CUsbPicCan *gateway;
	map<string,CCanMsgId> nodeMessages;		///< 11-bit CAN message ID's a node can understand.

};
#endif
